# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####


bl_info = {
    "name": "Sequencer In and Outpoints",
    "description": "Define in- and outpoints of your material in the file browser",
    "author": "Björn Sonnenschein",
    "version": (0, 3),
    "blender": (2, 63, 0),
    "location": "File Browser > Tools",
    "warning": "Experimental",
    "wiki_url": "None Yet"
                "None Yet",
    "tracker_url": "None"
                   "func=detail&aid=<number>",
    "category": "Sequencer"}

import bpy, os

############TO DO##########
#Falls auf edit range geklickt wird, aber schon eine Szene existiert mit dem Namen, einfach in diese wechseln.
 
#######Changelog########
##Let's keep track of changes if some people will develop this script independently from each other.

# Option to zoom to selection after entering edit range.
# Realigned ui

################################

scnType = bpy.types.Scene

def act_strip(context):
    try:
        return context.scene.sequence_editor.active_strip
    except AttributeError:
        return None


def detect_strip_type(filepath):
    imb_ext_image = [
    # IMG
    ".png",
    ".tga",
    ".bmp",
    ".jpg", ".jpeg",
    ".sgi", ".rgb", ".rgba",
    ".tif", ".tiff", ".tx",
    ".jp2",
    ".hdr",
    ".dds",
    ".dpx",
    ".cin",
    ".exr",
    # IMG QT
    ".gif",
    ".psd",
    ".pct", ".pict",
    ".pntg",
    ".qtif",
    ]

    imb_ext_movie = [
    ".avi",
    ".flc",
    ".mov",
    ".movie",
    ".mp4",
    ".m4v",
    ".m2v",
    ".m2t",
    ".m2ts",
    ".mts",
    ".mv",
    ".avs",
    ".wmv",
    ".ogv",
    ".dv",
    ".mpeg",
    ".mpg",
    ".mpg2",
    ".vob",
    ".mkv",
    ".flv",
    ".divx",
    ".xvid",
    ".mxf",
    ]

    imb_ext_audio = [
    ".wav",
    ".ogg",
    ".oga",
    ".mp3",
    ".mp2",
    ".ac3",
    ".aac",
    ".flac",
    ".wma",
    ".eac3",
    ".aif",
    ".aiff",
    ".m4a",
    ]

    extension = os.path.splitext(filepath)[1]
    extension = extension.lower()
    if extension in imb_ext_image:
        type = 'IMAGE'
    elif extension in imb_ext_movie:
        type = 'MOVIE'
    elif extension in imb_ext_audio:
        type = 'SOUND'
    else:
        type = None

    return type

#Boolean Custom Screen
BoolProperty = bpy.props.BoolProperty                              
scnType.custom_screen = BoolProperty( name="Custom Screen", 
                                     description = "Use a custom screen layout for range editing? ", 
                                     default=False )  
                                     
scnType.zoom = BoolProperty( name="Zoom", 
                                     description = "Zoom to the entire Clip after entering Edit Range? ", 
                                     default=False )  
#Is it the timeline scene?
BoolProperty = bpy.props.BoolProperty                              
scnType.timeline = BoolProperty( name="Timeline", 
                                     description = "Is this your actual timeline?", 
                                     default=False)                                       
                                     
#Define Screen to change to for editing range
StringProperty = bpy.props.StringProperty
editingrangestring = StringProperty(name="Editing Range Screen", description="The name of the screen layout you use for editing range", default="Video Editing" )
scnType.editing_range_screen = editingrangestring

#Define Screen to change to for editing
editingstring = StringProperty(name="Editing Screen", description="The name of the screen layout you use for editing", default="Video Editing" )
scnType.editing_screen = editingstring                                     

#Channel selector
IntProperty = bpy.props.IntProperty
intprop = IntProperty( name="Channel", description="Define in which channel the new strip will be inserted ", default=1, min=1, max=30, step=1)
scnType.channel = intprop

#Define the Path of the File the Scene belongs to.
StringProperty = bpy.props.StringProperty
sourcepath = StringProperty(name="Source Path", description="The Path of the File the Scene belongs to.", default="none" )
scnType.source_path = sourcepath


class FilePanel(bpy.types.Panel):    
    bl_space_type = "SEQUENCE_EDITOR"
    bl_region_type = "UI"
    bl_label = "Movie Manager"

    def draw(self, context):
        scn = bpy.context.scene
        layout = self.layout
        
        row = layout.row()
        col = row.column()  
        col.operator( "bpt.set_timeline" )         
        row.prop( scn, "channel" ) 
        
        row = layout.row()
        col = row.column() 
        
        col.prop( scn, "editing_screen" ) 
        col.prop( scn, "editing_range_screen" ) 
        
        row = layout.row()
        col = row.column() 
        row.prop( scn, "custom_screen" )   
        col.prop( scn, "zoom" )  
        
        row = layout.row()
        col = row.column() 
        
        col.operator( "bpt.edit_range" ) 
        row.operator( "bpt.switch_back_to_timeline" )
        
        row = layout.row()
        col = row.column() 
        col.operator( "bpt.insert_strip" )  
        
class Set_Timeline (bpy.types.Operator):
    bl_idname = "bpt.set_timeline"
    bl_label = "Set as Timeline"
    bl_description = "Set this scene as Timeline"    
    
    def invoke (self, context, event):
        for i in bpy.data.scenes:
            i.timeline = False 
            
        bpy.context.scene.timeline = True
        
        return {'FINISHED'}               
 
class Edit_Range_Operator(bpy.types.Operator):  
    
    bl_idname = "bpt.edit_range"
    bl_label = "Edit Range"
    bl_description = "Edit the Range of the selected clip in the File Browser. Use the new scene's Start and end Frame"
    
    def invoke (self, context, event):        
        
        #get scene
        for a in context.window.screen.areas:
            if a.type == 'FILE_BROWSER':
                params = a.spaces[0].params
                break
           
        try:
            params
            
        except:
            self.report({'ERROR_INVALID_INPUT'}, 'No visible File Browser')
            return {'CANCELLED'}

        if params.filename == '':
            self.report({'ERROR_INVALID_INPUT'}, 'No file selected')
            return {'CANCELLED'}

        path = params.directory + params.filename
        strip_type = detect_strip_type(params.filename)
        scene_exists = False
        scene_name = params.filename + "_Range"

        if strip_type in ('MOVIE'):
            
            for i in bpy.data.scenes:
                if i.source_path == path:
                    scene_exists = True
                    scene_name = i.name            

            editing_range_screen_name = bpy.context.scene.editing_range_screen
            editing_screen_name = bpy.context.scene.editing_screen
            custom_screen_bool = bpy.context.scene.custom_screen
            zoom_bool = bpy.context.scene.zoom
            
            if (scene_exists == True):                
                scene = bpy.data.scenes[scene_name]                 
                
                if (custom_screen_bool == True):                
                    for i in bpy.data.screens:
                        bpy.ops.screen.screen_set(delta=1)
                        
                        if (bpy.context.screen.name == editing_range_screen_name):
                            break
           
                    bpy.context.screen.scene = scene
                                            
                else:
                    bpy.context.screen.scene = scene                    
                
                bpy.context.scene.editing_range_screen = editing_range_screen_name   
                bpy.context.scene.editing_screen = editing_screen_name  
                bpy.context.scene.custom_screen = custom_screen_bool 
                bpy.context.scene.zoom = zoom_bool 
            
            else:
                new_scene = bpy.data.scenes.new(scene_name)                
                scene_name = new_scene.name                
                bpy.data.scenes[scene_name].source_path = path 
                
                timeline_scene = bpy.context.scene
                
                for i in bpy.data.scenes:
                    if i.timeline == True:
                        timeline_scene = i
                        break   
              
                new_scene.render.resolution_x = timeline_scene.render.resolution_x
                new_scene.render.resolution_y = timeline_scene.render.resolution_y
                new_scene.render.resolution_percentage = timeline_scene.render.resolution_percentage
                new_scene.render.fps  = timeline_scene.render.fps
                new_scene.frame_start = 0   
                            
                scene = bpy.data.scenes[scene_name]
                
                if (custom_screen_bool == True):                
                    for i in bpy.data.screens:
                        bpy.ops.screen.screen_set(delta=1)
                        
                        if (bpy.context.screen.name == editing_range_screen_name):
                            break
           
                    bpy.context.screen.scene = scene
                        
                else:
                    bpy.context.screen.scene = scene     
                
                bpy.context.scene.editing_range_screen = editing_range_screen_name   
                bpy.context.scene.editing_screen = editing_screen_name
                bpy.context.scene.custom_screen = custom_screen_bool
                bpy.context.scene.zoom = zoom_bool  
                   
                bpy.ops.sequencer.movie_strip_add(frame_start=0, filepath=path)
                bpy.context.scene.frame_end = bpy.context.scene.sequence_editor.active_strip.frame_final_duration           
                
        else:
            self.report({'ERROR_INVALID_INPUT'}, 'Invalid file format')
            return {'CANCELLED'}
        if (bpy.context.scene.zoom == True):
            zoom = bpy.ops.sequencer.view_selected()
        
        return {'FINISHED'}

class Switch_back_to_Timeline_Operator(bpy.types.Operator): 
     
    bl_idname = "bpt.switch_back_to_timeline"
    bl_label = "Get Back"
            
    def invoke(self, context, event ):  
        
        
        scene_name = bpy.context.scene.name
            
        for i in bpy.data.scenes:
            if i.timeline == True:
                scene_name = i.name
                break   
            
        editing_range_screen_name = bpy.context.scene.editing_range_screen
        editing_screen_name = bpy.context.scene.editing_screen
        custom_screen_bool = bpy.context.scene.custom_screen             
        scene = bpy.data.scenes[i.name]               
        i.zoom = bpy.context.scene.zoom
                     
        if (custom_screen_bool == True):         
            for i in bpy.data.screens:
                bpy.ops.screen.screen_set(delta=1)
                        
                if (bpy.context.screen.name == editing_screen_name):
                    break
           
            bpy.context.screen.scene = scene
                        
        else:
            bpy.context.screen.scene = scene     
                
        bpy.context.scene.editing_range_screen = editing_range_screen_name   
        bpy.context.scene.editing_screen = editing_screen_name
        bpy.context.scene.custom_screen = custom_screen_bool 
           
        return {'FINISHED'}    
    
class Insert_Strip(bpy.types.Operator):  
    
    bl_idname = "bpt.insert_strip"
    bl_label = "Insert selected Video"
    bl_description = "Insert the selected File into the Timeline"
    
    def invoke(self, context, event):
        for a in context.window.screen.areas:
            if a.type == 'FILE_BROWSER':
                params = a.spaces[0].params
                break
           
        try:
            params
            
        except:
            self.report({'ERROR_INVALID_INPUT'}, 'No visible File Browser')
            return {'CANCELLED'}

        if params.filename == '':
            self.report({'ERROR_INVALID_INPUT'}, 'No file selected')
            return {'CANCELLED'}

        path = params.directory + params.filename
        strip_type = detect_strip_type(params.filename)
        scene_exists = False

        if strip_type in ('MOVIE'):
            
            for i in bpy.data.scenes:
                if i.source_path == path:
                    scene_exists = True
                    scene_name = i.name
                    scene = i
                    break       
            
            #Get current frame
            current_frame = bpy.context.scene.frame_current
            channel = bpy.context.scene.channel
            
            
                     
            if (scene_exists == True):
                bpy.ops.sequencer.movie_strip_add(frame_start=current_frame, channel=channel, filepath=path)
                
                frame_start = bpy.context.scene.sequence_editor.active_strip.frame_start
                frame_end = bpy.context.scene.sequence_editor.active_strip.frame_final_end
                duration = bpy.context.scene.sequence_editor.active_strip.frame_duration
                end_offset = duration - scene.frame_end
                
                #Apply in and out points
                for i in bpy.context.selected_sequences:   
                    i.frame_final_start = frame_start + scene.frame_start
                    i.frame_final_end = frame_end - end_offset + 1
                    i.frame_start = frame_start - scene.frame_start       
            else:
                bpy.ops.sequencer.movie_strip_add(frame_start=current_frame, channel=channel, filepath=path)
                
            return {'FINISHED'}         
 
def register():
    bpy.utils.register_class( FilePanel )
    bpy.utils.register_class( Edit_Range_Operator ) 
    bpy.utils.register_class( Set_Timeline )
    bpy.utils.register_class( Switch_back_to_Timeline_Operator )
    bpy.utils.register_class( Insert_Strip )
 
 
def unregister():
    bpy.utils.register_class( FilePanel )
    bpy.utils.register_class( Edit_Range_Operator ) 
    bpy.utils.register_class( Set_Timeline )
    bpy.utils.register_class( Switch_back_to_Timeline_Operator)
    bpy.utils.register_class( Insert_Strip )
 
if __name__ == "__main__":
    register()
    
    
    
    


